<!-- -*- encoding: utf-8 -*- -->

Build Yliluoma's ordered dithering
==================================

2022/06/22

[Arbitrary-palette positional dithering algorithm](https://bisqwit.iki.fi/story/howto/dither/jy/)

上記ページで、任意のパレットを使って Ordered dithering をするアルゴリズムが紹介されている。

ソース群(.cpp)を、Windows10 x64 21H2 + MSYS2上でビルドできるか試してみた。


必要なもの
----------

ビルドには g++、make、libgd が必要。

```sh
pacman -S mingw-w64-x86_64-libgd
pacman -S mingw-w64-i686-libgd
```

```sh
$ g++ --version
g++.exe (Rev2, Built by MSYS2 project) 12.1.0

$ make --version
GNU Make 4.3
このプログラムは x86_64-pc-msys 用にビルドされました
```


コンパイル
----------

```sh
cd src
g++ yliluoma_ordered_dither_1a.cpp -fopenmp -o yliluoma_ordered_dither_1a.exe -lgd -lm
```

あるいは以下。

```sh
cd src
make
make install
```

make install を実行すると、生成された .exe を ../windows/ に移動する。

静的リンクをしたかったけど大量にエラーが出てダメだった。
libgd関連のシンボルが見つからないと言われる…。

とりあえずMSYS2上だけでも動いてくれれば動作確認ぐらいはできるだろう…
ぐらいの感じで作業してみた。

一応、windows/ に、動作に必要な .dll を全部入れておいたので、
MSYS2がインストールされてない環境でも .exe の動作確認ぐらいはできる…はず。たぶん。


ファイル一覧
------------

* src/yliluoma_ordered_dither_1a.cpp : 2色の組み合わせを評価して妥当な2色を選んでディザをかける。
* src/yliluoma_ordered_dither_1ba.cpp : 異なり過ぎる2色の組み合わせは除外して処理する。
* src/yliluoma_ordered_dither_1bb.cpp : 2色の差を CCIR 601 で求める。
* src/yliluoma_ordered_dither_1c.cpp : ループで結果を求めず数式で求めて高速化。ただし品質は落ちる。
* src/yliluoma_ordered_dither_1d.cpp : 3色のディザリングで処理する。
* src/yliluoma_ordered_dither_2a.cpp : ディザは2色の組み合わせという前提を捨てて考えた版。
* src/yliluoma_ordered_dither_2b.cpp : 2a のガンマ値を補正した版。
* src/yliluoma_ordered_dither_3a.cpp : 2b のアレンジ版。
* src/adobelike_pattern_dither.cpp : Photoshopに近い処理。


使い方
------

windowsディレクトリをカレントディレクトリにして、
以下を実行すれば画像が生成される。

* bash上なら ./test.sh を実行。
* DOS窓上なら test.bat を実行。

```sh
cd windows
./test.sh
```

あるいは、以下を打つ。

```sh
./yliluoma_ordered_dither_1a.exe scene.png scenedither1.png
./yliluoma_ordered_dither_1ba.exe scene.png scenedither2.png
./yliluoma_ordered_dither_1bb.exe scene.png scenedither4.png
./yliluoma_ordered_dither_1c.exe scene.png scenedither5.png
./yliluoma_ordered_dither_1d.exe scene.png scenedither6.png
./yliluoma_ordered_dither_2a.exe scene.png scenedither10.png
./yliluoma_ordered_dither_2b.exe scene.png scenedither10-gamma_rgb.png
./yliluoma_ordered_dither_3a.exe scene.png scenedither11-gamma.png
./adobelike_pattern_dither.exe scene.png scenedither12.png
```


Ubuntu上でビルド
----------------

Ubuntu Linux 20.04 LTS上でもビルドできるか試してみた。

```sh
sudo apt install libgd-dev
cd src
make -f Makefile.unix
make -f Makefile.unix install
```

画像生成は以下。

```sh
cd ../linux
./test.sh
```

